"""Basic layout components.
"""
import grok
from hurry import yui
from megrok.layout import Page, Layout, EditForm, DisplayForm, AddForm
from waeup.interfaces import IWAeUPObject
from waeup.viewlets import LeftSidebar
from z3c.flashmessage.interfaces import IMessageSource
from zope.component import queryUtility, ComponentLookupError
from zope.interface import Interface
# Conditional import. getSite moved to zope.site.hooks in recent grok versions.
try:
    from zope.site.hooks import getSite
except ImportError:
    from zope.app.component.hooks import getSite

grok.templatedir('templates')
    
class UtilityView(object):
    """A view mixin with useful methods.

    XXX: This class was copied verbatim from megrok.layout trunk. It
    can be imported with next release of megrok.layout, so that this
    piece can be removed.
    """
    def application_url(self, name=None):
        """Return the URL of the nearest site.
        """
        site = getSite()
        #if not site:
        #    raise ComponentLookupError("No site found.")
        return self.url(site, name)

    def flash(self, message, type='message'):
        """Send a short message to the user.
        """
        source = queryUtility(IMessageSource, name='session')
        if source is None:
            return None
        source.send(message, type)
        return True

class WAeUPLayout(Layout, UtilityView):
    """A megrok.layout.Layout with additional methods.
    """
    grok.baseclass()
    
class WAeUPPage(Page, UtilityView):
    """A megrok.layout page with additional methods.
    """
    grok.baseclass()

class WAeUPDisplayFormPage(DisplayForm, UtilityView):
    """A megrok.layout.DisplayForm with additional methods.
    """
    grok.baseclass()

class WAeUPEditFormPage(EditForm, UtilityView):
    """A megrok.layout.EditForm with additional methods.
    """
    grok.baseclass()

class WAeUPAddFormPage(AddForm,  UtilityView):
    """A megrok.layout.AddForm with additional methods.
    """
    grok.baseclass()

class LeftSidebarLink(grok.Viewlet):
    """ An entry on left sidebar.

    This is only a baseclass that won't be rendered actually. Deriving
    viewlets can override certain values and will be rendered with the
    values set here as default.
    """
    grok.baseclass()
    grok.viewletmanager(LeftSidebar)
    grok.context(Interface)
    grok.order(5)
    grok.require('waeup.manageUniversity')

    title = 'Text of link'
    @property
    def url(self):
        return 'URL to link to'
    
    def render(self):
        return u'<div class="portlet"><a href="%s">%s</a></div>' % (
            self.url, self.title)

class SiteLayout(WAeUPLayout):
    """ The general site layout.
    """
    grok.context(IWAeUPObject)

    def update(self):
        yui.reset_fonts_grids.need()
        yui.sam.need()
        yui.tabview.need()
        yui.layout.need()

